@@ -151,7 +151,9 @@ ServerOptions::ServerOptions()
     , health_reporter(NULL)
     , rtmp_service(NULL)
     , redis_service(NULL)
-    , bthread_tag(BTHREAD_TAG_INVALID) {
+    , bthread_tag(BTHREAD_TAG_INVALID)
+    , rpc_pb_message_factory(new DefaultRpcPBMessageFactory())
+    , ignore_eovercrowded(false) {
     if (s_ncore > 0) {
         num_threads = s_ncore + 1;
     }
@@ -443,9 +445,15 @@ Server::~Server() {
     _options.thrift_service = NULL;
 #endif
 
+    delete _options.baidu_master_service;
+    _options.baidu_master_service = NULL;
+
     delete _options.http_master_service;
     _options.http_master_service = NULL;
 
+    delete _options.rpc_pb_message_factory;
+    _options.rpc_pb_message_factory = NULL;
+
     delete _am;
     _am = NULL;
     delete _internal_am;
