@@ -21,9 +21,6 @@ StackTrace::StackTrace(const void* const* trace, size_t count) {
   count_ = count;
 }
 
-StackTrace::~StackTrace() {
-}
-
 const void *const *StackTrace::Addresses(size_t* count) const {
   *count = count_;
   if (count_)
@@ -31,6 +28,12 @@ const void *const *StackTrace::Addresses(size_t* count) const {
   return NULL;
 }
 
+size_t StackTrace::CopyAddressTo(void** buffer, size_t max_nframes) const {
+    size_t nframes = std::min(count_, max_nframes);
+    memcpy(buffer, trace_, nframes * sizeof(void*));
+    return nframes;
+}
+
 std::string StackTrace::ToString() const {
   std::stringstream stream;
 #if !defined(__UCLIBC__)
