@@ -29,6 +29,7 @@
 #include "butil/macros.h"                 // BAIDU_CACHELINE_ALIGNMENT
 #include "butil/scoped_lock.h"            // BAIDU_SCOPED_LOCK
 #include "butil/thread_local.h"           // thread_atexit
+#include "butil/memory/manual_constructor.h"
 #include <vector>
 
 #ifdef BUTIL_RESOURCE_POOL_NEED_FREE_ITEM_NUM
@@ -113,7 +114,7 @@ public:
     // items in the Block are only used by the thread.
     // To support cache-aligned objects, align Block.items by cacheline.
     struct BAIDU_CACHELINE_ALIGNMENT Block {
-        char items[sizeof(T) * BLOCK_NITEM];
+        ManualConstructor<T> items[BLOCK_NITEM];
         size_t nitem;
 
         Block() : nitem(0) {}
@@ -162,48 +163,52 @@ public:
         // which may include parenthesis because when T is POD, "new T()"
         // and "new T" are different: former one sets all fields to 0 which
         // we don't want.
-#define BAIDU_RESOURCE_POOL_GET(CTOR_ARGS)                              \
-        /* Fetch local free id */                                       \
-        if (_cur_free.nfree) {                                          \
+#define BAIDU_RESOURCE_POOL_GET(...)                                        \
+        /* Fetch local free id */                                           \
+        if (_cur_free.nfree) {                                              \
             const ResourceId<T> free_id = _cur_free.ids[--_cur_free.nfree]; \
-            *id = free_id;                                              \
-            BAIDU_RESOURCE_POOL_FREE_ITEM_NUM_SUB1;                   \
-            return unsafe_address_resource(free_id);                    \
-        }                                                               \
-        /* Fetch a FreeChunk from global.                               \
-           TODO: Popping from _free needs to copy a FreeChunk which is  \
-           costly, but hardly impacts amortized performance. */         \
-        if (_pool->pop_free_chunk(_cur_free)) {                         \
-            --_cur_free.nfree;                                          \
-            const ResourceId<T> free_id =  _cur_free.ids[_cur_free.nfree]; \
-            *id = free_id;                                              \
-            BAIDU_RESOURCE_POOL_FREE_ITEM_NUM_SUB1;                   \
-            return unsafe_address_resource(free_id);                    \
-        }                                                               \
-        /* Fetch memory from local block */                             \
-        if (_cur_block && _cur_block->nitem < BLOCK_NITEM) {            \
+            *id = free_id;                                                  \
+            BAIDU_RESOURCE_POOL_FREE_ITEM_NUM_SUB1;                         \
+            return unsafe_address_resource(free_id);                        \
+        }                                                                   \
+        /* Fetch a FreeChunk from global.                                   \
+           TODO: Popping from _free needs to copy a FreeChunk which is      \
+           costly, but hardly impacts amortized performance. */             \
+        if (_pool->pop_free_chunk(_cur_free)) {                             \
+            --_cur_free.nfree;                                              \
+            const ResourceId<T> free_id =  _cur_free.ids[_cur_free.nfree];  \
+            *id = free_id;                                                  \
+            BAIDU_RESOURCE_POOL_FREE_ITEM_NUM_SUB1;                         \
+            return unsafe_address_resource(free_id);                        \
+        }                                                                   \
+        T* p = NULL;                                                        \
+        auto ctor = [&](void* mem) {                                        \
+            p = new (mem) T(__VA_ARGS__);                                   \
+        };                                                                  \
+        /* Fetch memory from local block */                                 \
+        if (_cur_block && _cur_block->nitem < BLOCK_NITEM) {                \
             id->value = _cur_block_index * BLOCK_NITEM + _cur_block->nitem; \
-            T* p = new ((T*)_cur_block->items + _cur_block->nitem) T CTOR_ARGS; \
-            if (!ResourcePoolValidator<T>::validate(p)) {               \
-                p->~T();                                                \
-                return NULL;                                            \
-            }                                                           \
-            ++_cur_block->nitem;                                        \
-            return p;                                                   \
-        }                                                               \
-        /* Fetch a Block from global */                                 \
-        _cur_block = add_block(&_cur_block_index);                      \
-        if (_cur_block != NULL) {                                       \
+            (_cur_block->items + _cur_block->nitem)->InitBy(ctor);          \
+            if (!ResourcePoolValidator<T>::validate(p)) {                   \
+                p->~T();                                                    \
+                return NULL;                                                \
+            }                                                               \
+            ++_cur_block->nitem;                                            \
+            return p;                                                       \
+        }                                                                   \
+        /* Fetch a Block from global */                                     \
+        _cur_block = add_block(&_cur_block_index);                          \
+        if (_cur_block != NULL) {                                           \
             id->value = _cur_block_index * BLOCK_NITEM + _cur_block->nitem; \
-            T* p = new ((T*)_cur_block->items + _cur_block->nitem) T CTOR_ARGS; \
-            if (!ResourcePoolValidator<T>::validate(p)) {               \
-                p->~T();                                                \
-                return NULL;                                            \
-            }                                                           \
-            ++_cur_block->nitem;                                        \
-            return p;                                                   \
-        }                                                               \
-        return NULL;                                                    \
+            (_cur_block->items + _cur_block->nitem)->InitBy(ctor);          \
+            if (!ResourcePoolValidator<T>::validate(p)) {                   \
+                p->~T();                                                    \
+                return NULL;                                                \
+            }                                                               \
+            ++_cur_block->nitem;                                            \
+            return p;                                                       \
+        }                                                                   \
+        return NULL;                                                        \
  
 
         inline T* get(ResourceId<T>* id) {
@@ -212,12 +217,12 @@ public:
 
         template <typename A1>
         inline T* get(ResourceId<T>* id, const A1& a1) {
-            BAIDU_RESOURCE_POOL_GET((a1));
+            BAIDU_RESOURCE_POOL_GET(a1);
         }
 
         template <typename A1, typename A2>
         inline T* get(ResourceId<T>* id, const A1& a1, const A2& a2) {
-            BAIDU_RESOURCE_POOL_GET((a1, a2));
+            BAIDU_RESOURCE_POOL_GET(a1, a2);
         }
 
 #undef BAIDU_RESOURCE_POOL_GET
@@ -384,7 +389,7 @@ private:
 
     // Create a Block and append it to right-most BlockGroup.
     static Block* add_block(size_t* index) {
-        Block* const new_block = new(std::nothrow) Block;
+        Block* const new_block = new (std::nothrow) Block;
         if (NULL == new_block) {
             return NULL;
         }
