@@ -349,7 +349,7 @@ TEST_F(MultiDimensionTest, stats) {
     ASSERT_TRUE(my_madder.has_stats(labels_value3));
     ASSERT_FALSE(my_madder.has_stats(labels_value4));
 
-    my_madder.delete_stats();
+    my_madder.clear_stats();
     ASSERT_EQ(0, my_madder.count_stats());
     ASSERT_FALSE(my_madder.has_stats(labels_value1));
     ASSERT_FALSE(my_madder.has_stats(labels_value2));
